<?php

/**
* 01/06/2015
* @author Francis
*/

namespace Teknisa\GER\Controllers;

use \Zeedhi\Framework\DTO\Request;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Request\Filter;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Teknisa\GER\Modules\Auth\Services\AuthService;

class AuthController {

	protected $authService;

	public function __construct(AuthService $authService) {
		$this->authService = $authService;
	}



	public function checkLogin($request, $response) {
		$response->addDataSet(new DataSet ('user', $this->authService->getUserData()));
	}
	public function login($request, $response) {
		try {
			$credentials = $request->getFilterCriteria()->getConditions();
			$email = isset($credentials[0]['value']) ? $credentials[0]['value'] : null;
			$password = isset($credentials[1]['value']) ? $credentials[1]['value'] : null;
			$user = $this->authService->getAuthUser($email, $password);
			$response->addDataSet(new DataSet ('user', $user));
		} catch(\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function autenticar($request, $response) {
		try {
			$credentials = $request->getFilterCriteria()->getConditions();
			$email = isset($credentials[0]['value']) ? $credentials[0]['value'] : null;
			$password = isset($credentials[1]['value']) ? $credentials[1]['value'] : null;
			$user = $this->authService->autenticar($email, $password);
			$response->addDataSet(new DataSet ('user', $user));
		} catch(\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function requestNewPassword($request, $response) {
		try {
			$conditions = $request->getFilterCriteria()->getConditions();
			$email = isset($conditions[0]['value']) ? $conditions[0]['value'] : null;
			$url = isset($conditions[1]['value']) ? $conditions[1]['value'] : null;
			$this->authService->requestNewPassword($email, $url);
			$response->addDataSet(new DataSet ('success', array(true)));
		} catch(\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function resetPassword($request, $response) {
		try {
			$conditions = $request->getFilterCriteria()->getConditions();
			$newPassword = isset($conditions[0]['value']) ? $conditions[0]['value'] : null;
			$cryptography = isset($conditions[1]['value']) ? $conditions[1]['value'] : null;
			$this->authService->resetPassword($newPassword, $cryptography);
			$response->addDataSet(new DataSet ('success', array(true)));
		} catch(\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

}