ZeedhiChart.directive('zhGauge', function($timeout, maskEngine, ApplicationContext, ZhLogger, ZhUtil) {
	return {
		restrict: 'A',
		link: function($scope, $element) {

			var canvas = $element.find('canvas');
			var text = $element.find('span');

			function widget() {
				return $scope.widget;
			}

			/* Get functions */

			function getRow() {
				return widget().dataSource.data[0] || {};
			}

			function getGaugeValue() {
				return getRow()[widget().valueField];
			}

			function getLevels() {
				var levels = Util.clone(widget().gaugeLevels);
				if (hasRow()) {
					ApplicationContext.registerInstance('row', getRow());

					try {
						levels.forEach(function(level) {
							level.minValue = ZhUtil.evaluate(level.minValue);
							level.maxValue = ZhUtil.evaluate(level.maxValue);
						});
					} catch (e) {
						ZhLogger.error("Error while evaluating ZhStyle condition function", e);
					}

					ApplicationContext.removeInstance('row');
				}

				return levels;
			}

			function getGaugeText() {
				var fieldName = widget().descriptionField || widget().valueField;
				return getRow()[fieldName];
			}

			function getHintValue() {
				var fieldName = widget().hintField || widget().valueField;
				return getRow()[fieldName];
			}

			/* Set Functions */

			function setGaugeValue(value) {
				var fValue = parseFloat(value);
				var validValue;
				if (fValue >= gauge.maxValue) {
					validValue = gauge.maxValue;
				} else if (fValue <= gauge.minValue) {
					validValue = gauge.minValue + 1;
				} else {
					validValue = fValue;
				}
				gauge.set(validValue);
				setValueText();
				setHintText();
			}

			function setValueText() {
				var format = widget().valueFormat;
				if (maskEngine.hasFormater(format)) {
					var params = {};
					if (!Util.isUndefined(gauge.maxValue)) {
						params.maxValue = gauge.maxValue;
					}
					if (!Util.isUndefined(gauge.minValue)) {
						params.minValue = gauge.minValue;
					}
					if (!Util.isUndefined(widget().valuePrecision)) {
						params.precision = widget().valuePrecision;
					}
					
					var formatedValue = maskEngine.getFormater(format)(getGaugeText(), params);
					widget().valueText = formatedValue;
				} else {
					widget().valueText = getGaugeText();
				}
			}

			function setHintText() {
				var format = widget().hintFormat;
				if (format) {
					if (maskEngine.hasFormater(format)) {
						// If the given format is a valid maskEngine format uses it to format the values
						var formater = maskEngine.getFormater(format);
						var formatedValue = formater(getHintValue(), gauge.maxValue, gauge.minValue);
						var formatedMaxValue = formater(gauge.maxValue);
						widget().hintText = "(" + formatedValue + "/" + formatedMaxValue + ")";
					} else {
						// @TODO: Evaluate the format function and uses the returned value as hintText
						// var formatedText = ZhUtil.evaluate(widget().hintFormat);
						// widget().hintText = formatedText;
					}
				} else {
					widget().hintText = "(" + getHintValue() + "/" + gauge.maxValue + ")";
				}
			}

			/* Drawing functions */

			function setTextSize() {
				var size = Math.min($element.innerWidth() * 0.075, $element.innerHeight() * 0.1);
				text.css('font-size', (size > 12 ? size : 12) + 'px');
				text.css('line-height', (size > 12 ? size : 12) + 'px');
			}

			function setCanvasSize() {
				canvas.attr('width', $element.innerWidth());
				canvas.attr('height', $element.innerHeight() * 0.9);
			}

			function drawGauge() {
				gauge.value = getGaugeValue();
				setGaugeValue(gauge.value);
				setTextSize();
				setCanvasSize();
				gauge.setOptions(opts);
				gauge.update(true);
			}

			widget().redraw = drawGauge;

			widget().rebuildGauge = function() {
				canvas.remove();
				canvas = $("<canvas id="+widget().name+"></canvas>");

				$element.prepend(canvas);

				gauge = new Gauge(canvas[0]);
				gauge.minValue = widget().minValue || 0;
				gauge.maxValue = widget().maxValue;
				gauge.animationSpeed = 24;

				opts.levels = getLevels();
				drawGauge();
			};

			/* Auxiliar functions */

			var hasRow = function() {
				return !!widget().dataSource && !!widget().dataSource.data && !!widget().dataSource.data.length;
			};

			/* Gauge initial construction */

			var opts = {
				lines: 20,
				angle: 0,
				lineWidth: 0.45,
				limitMax: true,
				colorStart: '#666666',
				strokeColor: '#eeeeee',
				highDpiSupport: false,
				levels: getLevels(),
				pointer: {
					length: 0.70,
					strokeWidth: 0.035,
					color: '#333333'
				}
			};

			$element.click(function() {
				widget().setCurrentRow(getRow());
			});

			gauge = new Gauge(canvas[0]);
			gauge.minValue = widget().minValue || 0;
			gauge.maxValue = widget().maxValue;
			gauge.animationSpeed = 24;

			if (hasRow()) {
				gauge.value = getGaugeValue();
				setGaugeValue(gauge.value);
			} else {
				gauge.value = gauge.minValue;
				setGaugeValue(gauge.value);
			}

			$(window).resize(function() {
				if ($element.is(":visible")) {
					$timeout(function() {
						widget().rebuildGauge();
					},200, false);
				}
			});

			$scope.$watch(
				function() {
					return getGaugeValue();
				},
				function(e) {
					setTextSize();
					setCanvasSize();

					var value = getGaugeValue();
					if(value) {
						gauge.value = value;
						setGaugeValue(value);
						widget().rebuildGauge();
					}
				}
			);

			$scope.$watch(
				function() {
					return widget().dataSource.data && widget().dataSource.data.length;
				},
				function(e) {
					$timeout(function() {
						widget().rebuildGauge();
					}, 220, false);
				}
			);
		}
	};
});